#include "config_path.h"
#include "whereami.h"
#include <dirent.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/stat.h>

const char *config_path(void) {
  static char *path = NULL;

  if (path != NULL) return path;

  const char *suffix = "/calibration/";

  int dirname_length;
  int length = wai_getExecutablePath(NULL, 0, NULL);

  // We are totally going to let this memory leak all of one time until the program ends.
  path = (char*)malloc(length + 1 + strlen(suffix));
  if (path == NULL) {
    printf("Failed to malloc in %s:%d\n", __FILE__, __LINE__);
    exit(1);
  }

  wai_getExecutablePath(path, length, &dirname_length);
  path[dirname_length] = '\0'; // using dirname_length chops off the executable name and leaves the path

  strcat(path, suffix);

  DIR* dir = opendir(path);
  if (dir) {
    printf("Configration directory found: %s\n", path);
    closedir(dir);
  } else if (ENOENT == errno) {
    printf("Config directory %s not found.\n", path);
    exit(1);
  } else {
    printf("Failed to find config directory with errno %d.\n", errno);
    exit(1);
  }

  return path;
}

char *config_file_path(const char *filename) {
  char *path = malloc(strlen(config_path()) + strlen(filename) + 1);
  if (path == NULL) {
    printf("malloc failed at %s:%d\n", __FILE__, __LINE__);
    exit(1);
  }

  strcpy(path, config_path());
  strcpy(path + strlen(path), filename);

  struct stat stat_buffer;
  if (stat(path, &stat_buffer) != 0) {
    printf("Config file not found: %s\n", path);
    exit(1);
  }

  printf("  Config file found: %s\n", path);

  return path;
}
