/*
 * $URL$
 * 
 * Copyright (C) 2022 by Arizona State University and Mark Robinson.
 * All rights reserved.
 *
 * $Author$
 *   $Date$
 *    $Rev$
 *
 * (description of this file here)
 *
 */
#include "radiance.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include "config_path.h"
#include "special_pixels.h"

void read_csv(const char* filename, struct shadowcam_image* image, double *radiance_coeff) {
  FILE *file = fopen(filename, "rb");
  if (file == NULL) {
    printf("fopen of %s failed with errno %d\n", filename, errno);
    exit(1);
  }

  char buffer[1024];

  while(fgets(buffer, sizeof(buffer), file)) {
    if (strncmp(buffer, "#", 1) == 0)
      continue;

    double coeff_tmp[SHC_CHANNELS * 2];

    char *token = strtok(buffer, ",");
    if (token == NULL) {
      printf("Expected ',' in %s, loading CSV failed.\n", filename);
      exit(1);
    }

    for (int i=0; i < SHC_CHANNELS * 2; i++) {
      token = strtok(NULL, ",");
      if (token == NULL) {
        printf("Expected ',' in %s, loading CSV failed.\n", filename);
        exit(1);
      }
      coeff_tmp[i] = strtod(token, NULL);
    }

    for (uint8_t channel = 0; channel < SHC_CHANNELS; channel++)
      radiance_coeff[channel] = coeff_tmp[channel + SHC_CHANNELS * image->tdi_factor];
  }
}

void radiance(struct shadowcam_image *image) {
  puts("Radiance");

  double radiance_coeff[SHC_CHANNELS];

  puts("  Reading radiance coefficient CSV.");
  char *csv_path = config_file_path("radiance.csv");
  read_csv(csv_path, image, radiance_coeff);
  free(csv_path);

  puts("  Applying radiance coefficients.");

  for (uint32_t line=0; line < image->lines; line++)
    for (uint8_t channel=0; channel < SHC_CHANNELS; channel++)
      for (uint16_t pixel=0; pixel < SHC_AFE_WIDTH; pixel++)
        if (!is_special(image->data[data_index(line, channel, pixel)]))
          image->data[data_index(line, channel, pixel)] = image->data[data_index(line, channel, pixel)] / (image->line_rate_ms * radiance_coeff[channel]);
}
